Attribute VB_Name = "modListFolder"
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2017

Option Explicit

' **************** Const ****************
Private Const MAX_PATH As Long = 260&
Private Const INVALID_HANDLE_VALUE As Long = -1&
Private Const INVALID_FILE_ATTRIBUTES As Long = -1&
Private Const FILE_ATTRIBUTE_ARCHIVE As Long = &H20&
Private Const FILE_ATTRIBUTE_COMPRESSED As Long = &H800&
Private Const FILE_ATTRIBUTE_DIRECTORY As Long = &H10&
Private Const FILE_ATTRIBUTE_HIDDEN As Long = &H2&
Private Const FILE_ATTRIBUTE_NORMAL As Long = &H80&
Private Const FILE_ATTRIBUTE_READONLY As Long = &H1&
Private Const FILE_ATTRIBUTE_SYSTEM As Long = &H4&
Private Const FILE_ATTRIBUTE_TEMPORARY As Long = &H100&

' **************** Type ****************
Public Type FolderList
    FullPath As String
    FolderName As String
End Type

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime As FILETIME
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * MAX_PATH
    cAlternate As String * 14
End Type

' **************** Kernel32.dll ****************
Private Declare Function FindFirstFile Lib "kernel32" _
                         Alias "FindFirstFileA" ( _
                         ByVal lpFileName As String, _
                         ByRef lpFindFileData As WIN32_FIND_DATA) As Long

Private Declare Function FindNextFile Lib "kernel32" _
                         Alias "FindNextFileA" ( _
                         ByVal hFindFile As Long, _
                         ByRef lpFindFileData As WIN32_FIND_DATA) As Long

Private Declare Function FindClose Lib "kernel32" ( _
                         ByVal hFindFile As Long) As Long
                         
Private Declare Function GetFileAttributes Lib "kernel32" _
                         Alias "GetFileAttributesA" ( _
                         ByVal lpFileName As String) As Long

Public Function ListFolder(ByVal Root As String, ByRef FolderList() As FolderList) _
    As Long

    Dim lItem As Long
    Dim lCount As Long
    Dim FolderToDelete() As String

    ReDim FolderList(0)

    FolderList(0).FullPath = Root

    If ListAllFolderInRoot(FolderList(0).FullPath, FolderList) = True Then

        ListFolder = UBound(FolderList)

    End If

End Function

' ist eine Datei vorhanden
Public Function FileExists(ByVal sFileName As String) As Boolean

    Dim dwAttributes As Long

    dwAttributes = GetFileAttributes(sFileName)

    If dwAttributes = INVALID_FILE_ATTRIBUTES Then

        FileExists = False

    ElseIf CBool(dwAttributes And FILE_ATTRIBUTE_DIRECTORY) Then

        FileExists = False

    Else

        FileExists = True

    End If

End Function


Public Function DirExists(ByVal Root As String) As Boolean

    Dim File As String
    Dim hFile As Long
    Dim FD As WIN32_FIND_DATA

    ' Backslash ergnzen
    If Right$(Root, 1) <> "\" Then Root = Root & "\"

    ' Die erste Datei suchen
    hFile = FindFirstFile(Root & "*.*", FD)

    ' Es konnte nichts gefunden werden
    If hFile = INVALID_HANDLE_VALUE Then

        DirExists = False

        Exit Function

    End If

    ' Den Dateinamen extrahieren
    File = Left$(FD.cFileName, InStr(FD.cFileName, ChrW$(0)) - 1)

    ' Ist die Datei ein Verzeichnis?
    If (FD.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = _
        FILE_ATTRIBUTE_DIRECTORY Then

        DirExists = True

    End If

    ' Suchhandle wieder freigeben - Suche beenden
    Call FindClose(hFile)

End Function

Private Function ListAllFolderInRoot(ByVal Root As String, ByRef FolderInRoot() As _
    FolderList) As Boolean

    Dim File As String
    Dim hFile As Long
    Dim FD As WIN32_FIND_DATA

    ' Backslash ergnzen
    If Right$(Root, 1) <> "\" Then Root = Root & "\"

    ' Die erste Datei suchen
    hFile = FindFirstFile(Root & "*.*", FD)

    ' Es konnte nichts gefunden werden
    If hFile = INVALID_HANDLE_VALUE Then

        ListAllFolderInRoot = False

        Exit Function

    End If

    ' Fr jede Datei
    Do

        ' Den Dateinamen extrahieren
        File = Left$(FD.cFileName, InStr(FD.cFileName, ChrW$(0)) - 1)

        ' Ist die Datei ein Verzeichnis?
        If (FD.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = _
            FILE_ATTRIBUTE_DIRECTORY Then

            ' . und .. ignorieren
            If (File <> ".") And (File <> "..") Then

                ReDim Preserve FolderInRoot(UBound(FolderInRoot) + 1)

                FolderInRoot(UBound(FolderInRoot)).FullPath = Root & File
                FolderInRoot(UBound(FolderInRoot)).FolderName = File

            End If

        End If

        ' Nchste Datei suchen
    Loop While FindNextFile(hFile, FD)

    ' Suchhandle wieder freigeben - Suche beenden
    Call FindClose(hFile)

    ListAllFolderInRoot = True

End Function

