Attribute VB_Name = "modRegistry"
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2017

Option Explicit

' **************** Const ****************
Private Const ERROR_SUCCESS As Long = 0&

Private Const HKEY_CURRENT_USER As Long = &H80000001

Private Const STANDARD_RIGHTS_ALL As Long = &H1F0000
Private Const KEY_QUERY_VALUE As Long = &H1
Private Const KEY_SET_VALUE As Long = &H2
Private Const KEY_CREATE_SUB_KEY As Long = &H4
Private Const KEY_ENUMERATE_SUB_KEYS As Long = &H8
Private Const KEY_NOTIFY As Long = &H10
Private Const KEY_CREATE_LINK As Long = &H20
Private Const SYNCHRONIZE As Long = &H100000

Private Const KEY_ALL_ACCESS As Long = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or _
    KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or _
    KEY_CREATE_LINK) And (Not SYNCHRONIZE))

Private Const ERROR_NO_MORE_ITEMS As Long = 259&

Private Const REG_NONE As Long = 0
Private Const REG_SZ As Long = 1
Private Const REG_EXPAND_SZ As Long = 2
Private Const REG_BINARY As Long = 3
Private Const REG_DWORD As Long = 4
Private Const REG_DWORD_LITTLE_ENDIAN As Long = 4
Private Const REG_DWORD_BIG_ENDIAN As Long = 5
Private Const REG_LINK As Long = 6
Private Const REG_MULTI_SZ As Long = 7
Private Const REG_RESOURCE_LIST As Long = 8
Private Const REG_FULL_RESOURCE_DESCRIPTOR As Long = 9
Private Const REG_RESOURCE_REQUIREMENTS_LIST As Long = 10

' **************** Type ****************
Public Type RegDWord
    Name As String
    Value As Currency
End Type

' **************** Advapi32.dll ****************
Private Declare Function RegOpenKeyEx Lib "advapi32" _
                         Alias "RegOpenKeyExA" ( _
                         ByVal hKey As Long, _
                         ByVal lpSubKey As String, _
                         ByVal ulOptions As Long, _
                         ByVal samDesired As Long, _
                         ByRef phkResult As Long) As Long

Private Declare Function RegEnumKey Lib "advapi32" _
                         Alias "RegEnumKeyA" ( _
                         ByVal hKey As Long, _
                         ByVal dwIndex As Long, _
                         ByVal lpName As String, _
                         ByVal cbName As Long) As Long

Private Declare Function RegQueryValue Lib "advapi32" _
                         Alias "RegQueryValueA" ( _
                         ByVal hKey As Long, _
                         ByVal lpSubKey As String, _
                         ByVal lpValue As String, _
                         ByRef lpcbValue As Long) As Long

Private Declare Function RegCloseKey Lib "advapi32" ( _
                         ByVal hKey As Long) As Long

Private Declare Function RegEnumValue Lib "advapi32" _
                         Alias "RegEnumValueA" ( _
                         ByVal hKey As Long, _
                         ByVal dwIndex As Long, _
                         ByVal lpValueName As String, _
                         ByRef lpcbValueName As Long, _
                         ByVal lpReserved As Long, _
                         ByRef lpType As Long, _
                         ByRef lpData As Byte, _
                         ByRef lpcbData As Long) As Long

' **************** Kernel32.dll ****************
Private Declare Function ExpandEnvironmentStrings Lib "Kernel32" _
                         Alias "ExpandEnvironmentStringsA" ( _
                         ByVal lpSrc As String, _
                         ByVal lpDst As String, _
                         ByVal nSize As Long) As Long

Private Function CutNulls(ByVal strNull As String) As String

    CutNulls = Mid$(strNull, 1, InStr(strNull, Chr$(0)) - 1)

End Function

Private Function GetRegExpandSZ(ByVal strEnv As String) As String

    Dim strBuffer As String
    Dim lngBufSize As Long

    '   Gre des bentigten Speichers ermitteln
    strBuffer = vbNullString
    lngBufSize = Len(strBuffer)
    lngBufSize = ExpandEnvironmentStrings(strEnv, ByVal 0&, lngBufSize)

    '   entsprechend groen Speicher bereitstellen
    strBuffer = Space$(lngBufSize + Len(vbNullChar))
    lngBufSize = Len(strBuffer)

    '   Daten holen
    lngBufSize = ExpandEnvironmentStrings(strEnv, strBuffer, lngBufSize)

    '   NullChar wegschneiden
    lngBufSize = InStr(1&, strBuffer, vbNullChar)

    If lngBufSize <> 0& Then

        strBuffer = Left$(strBuffer, lngBufSize - 1&)

    End If

    '   Fertig
    GetRegExpandSZ = strBuffer

End Function

' Get the key information for this key and its subkeys.
Private Function GetKeyInfo(ByVal section As Long, ByVal key_name As String) As _
    String

    Dim subkeys As Collection
    Dim subkey_values As Collection
    Dim subkey_num As Integer
    Dim subkey_name As String
    Dim subkey_value As String
    Dim length As Long
    Dim hKey As Long
    Dim txt As String
    Dim subkey_txt As String
    Dim value_num As Long
    Dim value_name_len As Long
    Dim value_name As String
    Dim reserved As Long
    Dim value_type As Long
    Dim value_string As String
    Dim value_data(1 To 1024) As Byte
    Dim value_data_len As Long
    Dim i As Integer
    Dim tRegDWord() As RegDWord

    Set subkeys = New Collection
    Set subkey_values = New Collection

    ' Open the key.
    If RegOpenKeyEx(section, key_name, 0&, KEY_ALL_ACCESS, hKey) <> ERROR_SUCCESS _
        Then

        MsgBox "Error opening key."

        Exit Function

    End If

    ' Enumerate the key's values.
    value_num = 0


    Do

        value_name_len = 1024
        value_name = Space$(value_name_len)
        value_data_len = 1024

        If RegEnumValue(hKey, value_num, value_name, value_name_len, 0, _
            value_type, value_data(1), value_data_len) <> ERROR_SUCCESS Then Exit _
            Do

        value_name = Left$(value_name, value_name_len)

        If Len(value_name) = 0 Then value_name = "[Standard]"

        Select Case value_type

        Case REG_DWORD

            If value_num = 0 Then

                ReDim tRegDWord(value_num)

            Else

                ReDim Preserve tRegDWord(value_num)

            End If

            value_string = "&H" & Format$(Hex$(value_data(4)), "00") & Format$( _
                Hex$(value_data(3)), "00") & Format$(Hex$(value_data(2)), "00") & _
                Format$(Hex$(value_data(1)), "00")

            txt = txt & value_name & " = " & value_string & vbCrLf

        End Select

        value_num = value_num + 1

    Loop

    ' Enumerate the subkeys.
    subkey_num = 0

    Do

        ' Enumerate subkeys until we get an error.
        length = 256
        subkey_name = Space$(length)

        If RegEnumKey(hKey, subkey_num, subkey_name, length) <> ERROR_SUCCESS Then _
            Exit Do

        subkey_num = subkey_num + 1

        subkey_name = Left$(subkey_name, InStr(subkey_name, Chr$(0)) - 1)
        subkeys.Add subkey_name

        ' Get the subkey's value.
        length = 256
        subkey_value = Space$(length)

        If RegQueryValue(hKey, subkey_name, subkey_value, length) <> ERROR_SUCCESS _
            Then

            subkey_values.Add "Error"

        Else

            ' Remove the trailing null character.
            subkey_value = Left$(subkey_value, length - 1)
            subkey_values.Add subkey_value

        End If

    Loop

    ' Close the key.
    If RegCloseKey(hKey) <> ERROR_SUCCESS Then

        MsgBox "Error closing key."

    End If

    ' Recursively get information on the keys.
    For subkey_num = 1 To subkeys.Count

        subkey_txt = GetKeyInfo(section, key_name & "\" & subkeys(subkey_num))

    Next subkey_num

    GetKeyInfo = txt

End Function

Public Function GetVolumeRegRecycleBin(ByVal sVolumeGuid As String) As RegDWord()

    Dim KeyString As String
    Dim Keys() As String
    Dim Key() As String
    Dim tRegDWord() As RegDWord
    Dim lItem As Long
    Dim lCount As Long

    KeyString = _
        "Software\Microsoft\Windows\CurrentVersion\Explorer\BitBucket\Volume\"

    KeyString = KeyString & sVolumeGuid

    KeyString = GetKeyInfo(HKEY_CURRENT_USER, KeyString)

    Keys = Split(KeyString, vbCrLf)

    lCount = UBound(Keys)

    ReDim tRegDWord(lCount - 1)

    For lItem = 0 To lCount - 1

        Key = Split(Keys(lItem), " = ")

        tRegDWord(lItem).Name = Trim$(Key(0))
        tRegDWord(lItem).Value = CCur(Trim$(Key(1)))

    Next

    GetVolumeRegRecycleBin = tRegDWord

End Function


